# -*- coding: utf-8 -*-
# author: NhanDD3 <hp.duongducnhan@gmail.com>

import logging
import os
import json
from typing import Callable
from pathlib import Path
from logging.handlers import TimedRotatingFileHandler
from pythonjsonlogger import jsonlogger


def json_translate_obj(obj):
    # for example, serialize a custom object
    # if isinstance(obj, MyClass):
    #     return {"special": obj.special}
    return {'obj': str(obj)}


def setup_time_rotation_logger(
    name, 
    level, 
    log_file_dir=None, 
    format_json=False, 
    json_translator=json_translate_obj
):
    if not log_file_dir:
        log_file_dir = os.path.join(Path.home(), "cmk-tools", "logs")

    if not os.path.isdir(log_file_dir):
        os.makedirs(log_file_dir, exist_ok=True)
    
    log_file_path = os.path.join(log_file_dir, f"{name}.log")
    print("log files will be saved at:", log_file_path)

    logger = logging.getLogger(f'cmk-tools.{name}')
    logger.setLevel(logging.DEBUG)

    handler = TimedRotatingFileHandler(
        log_file_path, when="midnight", interval=1, backupCount=7
    )

    if format_json:
        formatter = jsonlogger.JsonFormatter(
            json_encoder=json.JSONEncoder,
            json_default=json_translator,
        )
    else:
        formatter = logging.Formatter("%(asctime)s - %(name)s - %(levelname)s - %(message)s")

    handler.setFormatter(formatter)
    handler.setLevel(level)
    logger.addHandler(handler)
    return logger



import sys
import os
import logging
from typing import Callable
from .utils import replace_domain
from .status import OK, WARNING, CRITICAL, UNKNOWN
import requests


# SCC_USE_TEST_SVR = 'http://testserver/'
SCC_USE_TEST_SVR = os.getenv("SCC_USE_TEST_SERVER")
if SCC_USE_TEST_SVR and not 'http' in SCC_USE_TEST_SVR:
    raise ValueError("SCC_USE_TEST_SERVER must be a valid URL")


def _terminate_check(status, msg):
    # -----------------------------------------   
    #   DO NOT MODIFY THIS FUNCTION
    # -----------------------------------------    
    print(f"{status} - {msg}")
    if status == OK:
        sys.exit(0)
    elif status == WARNING:
        sys.exit(1)
    elif status == CRITICAL:
        sys.exit(2)
    else:
        sys.exit(3)

  
def make_request(
    plugin_id: str,
    url, 
    method: str, 
    headers: dict = {}, 
    data:dict = {}, 
    verify=False, 
    terminate_check: Callable =_terminate_check,
    logger_name: str = "cmk-tools"
): 
    logger = logging.getLogger(logger_name)
    if not callable(terminate_check):
        raise ValueError("terminate_check must be a function")

    if method.upper() not in ['GET', 'POST', 'PUT', 'DELETE']:
        return terminate_check(UNKNOWN, f"request API method >>{method}<< not supported")
    
    response = None
    try:
        if SCC_USE_TEST_SVR:
            new_url = replace_domain(url, SCC_USE_TEST_SVR)
            headers = {
                **headers, 
                'X-Plugin-Id': plugin_id
            }
            response = requests.request('POST', new_url, headers=headers, data=data, verify=verify)
        else:
            response = requests.request(method, url, headers=headers, data=data, verify=verify)
        response.raise_for_status()
        
    except requests.exceptions.ConnectTimeout:
        logger.warning(f"Timeout when connect to {url}", api_resp=response)
        terminate_check(UNKNOWN, f"Timeout when connect to {url}")
    except requests.exceptions.ReadTimeout:
        logger.warning(f"Timeout when connect to {url}", api_resp=response)
        terminate_check(UNKNOWN, f"Timeout when read data from {url}")
    except Exception as e:
        logger.critical(f"Timeout when connect to {url}", api_resp=response)
        terminate_check(UNKNOWN, f"Error when request to {url} with error: {e}")
    finally:
        return response


# -*- coding: utf-8 -*-
# author: NhanDD3 <hp.duongducnhan@gmail.com>


__version__ = '1.1.1'


import logging

from .logger import setup_time_rotation_logger
from .redis_semaphore import (
    RedisSemaphore,
    run_with_semaphore,
    run_with_semaphore_decorator,
)

logger = setup_time_rotation_logger(
    "redis_semaphore",
    logging.DEBUG,
    format_json=True,
)


__all__ = [
    "RedisSemaphore",
    "run_with_semaphore",
    "run_with_semaphore_decorator",
]
